{{#multiple}}
<div>
  {{#previous}}
  <a href="{{previous}}">
    Prev {{per_page}}
  </a>
  {{/previous}}
  <h3>Page {{page}} of {{pages}}</h3>
  {{#next}}
  <a href="{{next}}">
    Next {{per_page}}
  </a>
  {{/next}}
</div>
{{/multiple}}
