<p>
  {{#pages_info}}{{full_path}}{{/pages_info}}
</p>
<p>
  {{#pages_info}}{{path}}{{/pages_info}}
</p>
<p>
  {{rt}}/
</p>
<p>
  {{svc}}/
</p>
<p>
  {{version}}
</p>
<p>
  {{>csrf}}
</p>
<p>
  {{#page_config}}{{url}}{{/page_config}}
</p>
<p>
  {{#wq_config.pages.item.can_change}}
  Can Edit Items
  {{/wq_config.pages.item.can_change}}
</p>
